@@ -227,9 +227,11 @@ struct e1inp_line {
 	/* file name and file descriptor of pcap for this line */
 	char *pcap_file;
 	int pcap_fd;
+
+	unsigned int connect_timeout;
 };
 #define e1inp_line_ipa_oml_ts(line) (&line->ts[0])
-#define e1inp_line_ipa_rsl_ts(line, trx_id) (&line->ts[1 + (trx_id)])
+#define e1inp_line_ipa_rsl_ts(line, trx_id) (((1 + (trx_id)) < NUM_E1_TS) ? (&line->ts[1 + (trx_id)]) : NULL)
 
 /* SS_L_INPUT signals */
 enum e1inp_signal_input {
@@ -336,7 +338,8 @@ int e1inp_ipa_bts_rsl_connect(struct e1inp_line *line,
 
 int e1inp_ipa_bts_rsl_connect_n(struct e1inp_line *line,
 			      const char *rem_addr, uint16_t rem_port,
-			      uint8_t trx_id);
+			      uint8_t trx_nr);
+int e1inp_ipa_bts_rsl_close_n(struct e1inp_line *line, uint8_t trx_nr);
 
 void e1inp_sign_link_destroy(struct e1inp_sign_link *link);
 int e1inp_line_update(struct e1inp_line *line);
@@ -362,4 +365,7 @@ struct input_signal_data {
 int abis_sendmsg(struct msgb *msg);
 int abis_rsl_sendmsg(struct msgb *msg);
 
+int e1inp_ts_send_raw(struct e1inp_ts *ts, struct msgb *msg);
+int e1inp_ts_send_hdlc(struct e1inp_ts *ts, struct msgb *msg);
+
 #endif /* _E1_INPUT_H */
