@@ -14,10 +14,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 /*! \file osmo_ortp.c
@@ -266,7 +262,7 @@ static int osmo_rtp_socket_fdreg(struct osmo_rtp_socket *rs)
 	return 0;
 }
 
-static void create_payload_types()
+static void create_payload_types(void)
 {
 	PayloadType *pt;
 
